//234567890123456789012345678901234567890123456789012345678
import greenfoot.*;

/**
 * The purpose of this class is to encapsulate a pair of
 * static top-level classes from which a RightArrow object
 * and a LeftArrow object can be instantiated.  Because
 * objects of these two classes are intended to work in
 * concert, and because they need to share some 
 * information, the two classes were encapsulated inside
 * this class as static top-level classes.
 * 
 * This class also maintains the information that objects
 * of the RightArrow and LeftArrow classes need to share.
 * 
 * @author Dick Baldwin 
 * @version July 6, 2008
 */
public class ArrowButtonManager02 extends Actor{
  //The following variables are used to keep track of how
  // many times the user has scrolled the scene to the
  // right and to the left. They are also used to determine
  // when the user has scrolled to the limit in either 
  // direction.  Set the limit values to one less than the
  // number of scenes in the world.
  protected static int rightClickCnt = 0;
  protected final static int rightClickLim = 2;
  protected final static int leftClickLim = 2;
  protected static int leftClickCnt = leftClickLim;
  
  //The following array stores references to GreenfootImage
  // objects that are used to set the background of the
  // world each time the user clicks one of the buttons.
  protected final static GreenfootImage[] backgrounds = {
                   new GreenfootImage("bathroom-tile.gif"),
                   new GreenfootImage("brick.jpg"),
                   new GreenfootImage("burlap.jpg")};

  //Note that this class doesn't need a constructor or an
  // act method.

  //Here is the beginning of a pair of static top-level
  // classes.
  
//=======================================================//
/**
 * This is a class from which a "right arrow" button can
 * be constructed.  While the button may be useful for a
 * variety of purposes, the primary intended purpose is to
 * scroll a world right and left through several different 
 * background scenes, much as you might see in a children's
 * educational game where the child can progress forward 
 * and backwards through a sequence of different "worlds."
 * 
 * Three different images are used to represent the button.
 * All three images consist of a filled blue oval with a
 * cutout of an arrow pointing to the right.  The NEUTRAL
 * version of the button has a transparent cutout.  The
 * "GO" version has the cutout colored green.  The "STOP"
 * version has the cutout colored red.
 * 
 * The neutral version is displayed whenever the mouse is
 * not over the button.  The green version is displayed 
 * when the user points to the button with the mouse and
 * there are more worlds to the right for the user to
 * explore.  The red version is displayed when the user
 * points to the button with the mouse and there are no 
 * more worlds to the right to be explored.
 * 
 * Clicking a green button with the mouse causes the next
 * scene to the right to be displayed in the world.  
 * Clicking a red button has no effect at all.
 * 
 * Variables and constants in the containing class named
 * ArrowButtonManager02 are used to track the number
 * of times the user has scrolled to the right along with
 * the limit on the number of allowable scrolls to the 
 * right
 * 
 * An array in the superclass is used to maintain 
 * references to a set of GreenfootImage objects that are
 * used to set the background for each of the scenes.
 * 
 * In the ideal case, only the data in the containing class
 * needs to be changed to use this button with worlds 
 * having different numbers of background scenes and 
 * different actual background images.
 * 
 * An object of this class is intended to be used in
 * conjunction with an object of a companion class named
 * LeftArrow.  For that reason, this class and the 
 * companion class were encapsulated inside the superclass
 * as static top-level classes.
 * 
 * @author Dick Baldwin 
 * @version July 6, 2008
 */
public static class RightArrow 
                              extends ArrowButtonManager02{
  GreenfootImage goRightArrow;
  GreenfootImage stopRightArrow;
  GreenfootImage neutralRightArrow;
  
  public RightArrow(){//constructor
    //Construct and scale the three GreenfootImage objects
    // that are used to represent the button in the world.
    goRightArrow = new GreenfootImage("gorightarrow.png");
    goRightArrow.scale(goRightArrow.getWidth()/2,
                       goRightArrow.getHeight()/2);
    stopRightArrow = 
                  new GreenfootImage("stoprightarrow.png");
    stopRightArrow.scale(stopRightArrow.getWidth()/2,
                         stopRightArrow.getHeight()/2);
    neutralRightArrow = 
               new GreenfootImage("neutralrightarrow.png");
    neutralRightArrow.scale(neutralRightArrow.getWidth()/2,
                          neutralRightArrow.getHeight()/2);
                          
    //Set the default image on the button.
    setImage(neutralRightArrow);
  }//end constructor

  public void act(){
    //Cause the image representing the button to return to
    // the neutral version when the mouse is not over the
    // button.
    if(Greenfoot.mouseMoved(getWorld())){
      setImage(neutralRightArrow);
    }//end if
    
    //Cause the cutout arrow portion of the button to
    // change from transparent to either red or green
    // depending on whether the user has already scrolled
    // to the right as far as is allowed whenever the
    // user points the mouse at the button.
    if(Greenfoot.mouseMoved(this) && 
                          (rightClickCnt < rightClickLim)){
      //Set the image to the green button
      setImage(goRightArrow);
    }else{
      if(Greenfoot.mouseMoved(this) && 
                         (rightClickCnt >= rightClickLim)){
        //Set the image to the red button.
        setImage(stopRightArrow);
      }//end if
    }//end else

    //When the button is clicked and the user hasn't 
    // already scrolled as far to the right as is allowed
    // (meaning the button will turn green), increment and 
    // decrement the counters that are maintained in the 
    // superclass named ArrowButtonManager02. Ignore the
    // click if the user has already scrolled as far to the
    // right as is allowed.
    //Also use the new value of the counter to set the
    // background of the world to the correct background
    // image.
    if(Greenfoot.mouseClicked(this) && 
                          (rightClickCnt < rightClickLim)){
      rightClickCnt++;
      leftClickCnt--;
      //Add code here to completely populate the new scene
      // in addition to changing the background.
      getWorld().setBackground(backgrounds[rightClickCnt]);
      
      //Cause the arrow to turn red at the limit.
      if(rightClickCnt >= backgrounds.length - 1){
        setImage(stopRightArrow);
      }//end if
      
    }//end if
  }//end act
}//end static top-level class named RightArrow
//=======================================================//

/**
 * See the description of the class named RightArrow. This
 * class is similar to that one except that the arrow
 * button instantiated from this class points to the left 
 * instead of to the right.
 * 
 * @author Dick Baldwin 
 * @version July 6, 2008
 */
public static class LeftArrow extends ArrowButtonManager02{
  GreenfootImage goLeftArrow;
  GreenfootImage stopLeftArrow;
  GreenfootImage neutralLeftArrow;
  
  public LeftArrow(){//constructor
    goLeftArrow = new GreenfootImage("goleftarrow.png");
    goLeftArrow.scale(goLeftArrow.getWidth()/2,
                      goLeftArrow.getHeight()/2);
    stopLeftArrow = 
                   new GreenfootImage("stopleftarrow.png");
    stopLeftArrow.scale(stopLeftArrow.getWidth()/2,
                        stopLeftArrow.getHeight()/2);
    neutralLeftArrow = 
                new GreenfootImage("neutralleftarrow.png");
    neutralLeftArrow.scale(neutralLeftArrow.getWidth()/2,
                           neutralLeftArrow.getHeight()/2);
    
    setImage(neutralLeftArrow);
  }//end constructor

  public void act(){
    if(Greenfoot.mouseMoved(getWorld())){
      setImage(neutralLeftArrow);
    }//end if
    
    if(Greenfoot.mouseMoved(this) && 
                            (leftClickCnt < leftClickLim)){
      setImage(goLeftArrow);
    }else{
      if(Greenfoot.mouseMoved(this) && 
                           (leftClickCnt >= leftClickLim)){
        setImage(stopLeftArrow);
      }//end if
    }//end else

    if(Greenfoot.mouseClicked(this) && 
                            (leftClickCnt < leftClickLim)){
      leftClickCnt++;
      rightClickCnt--;
      getWorld().setBackground(backgrounds[rightClickCnt]);
      
      //Cause the arrow to turn red at the limit.
      if(leftClickCnt >= backgrounds.length - 1){
        setImage(stopLeftArrow);
      }//end if
      
    }//end if
  }//end act
}//end static top-level class named LeftArrow
//=======================================================//

}//end class ArrowButtonManager02
